@@ -1,6 +1,6 @@
 package DotCloud::Environment;
 {
-  $DotCloud::Environment::VERSION = '0.7.0_02';
+  $DotCloud::Environment::VERSION = '0.7.0_03';
 }
 
 # ABSTRACT: easy handling of environment in dotcloud
@@ -12,12 +12,14 @@ use English qw( -no_match_vars );
 use Storable qw< dclone >;
 
 our $main_file_path = '/home/dotcloud/environment.json';
+our $main_dotcloud_code_dir = '/home/dotcloud/code';
 my @application_keys = qw< environment project service_id service_name >;
 
 sub new {
    my $package = shift;
    my %params = (@_ > 0 && ref $_[0]) ? %{ $_[0] } : @_;
    my $self = bless { _params => \%params, _envfor => {} }, $package;
+   $self->{backtrack} = $params{backtrack} if exists $params{backtrack};
    $self->load() unless $params{no_load};
    return $self;
 }
@@ -149,9 +151,67 @@ sub _get_environment {
       if exists $self->{_params}{fallback_string};
    return _slurp($self->{_params}{fallback_file})
       if exists $self->{_params}{fallback_file};
+
+   return unless $params{backtrack} || $self->{backtrack};
+
+   # We will backtrack from three starting points:
+   # * the "root" directory for the application, i.e
+   #   what in dotCloud is /home/dotcloud/code
+   # * the current working directory
+   # * the directory containing the file that called us
+   my $code_dir = find_code_dir(n => 1);
+
+   require Cwd;
+   require File::Basename;
+   require File::Spec;
+   for my $path ($code_dir, Cwd::cwd(), File::Basename::dirname((caller())[1])) {
+      my ($volume, $directories) = File::Spec->splitpath($path, 'no-file');
+      my @directories = File::Spec->splitdir($directories);
+      while (@directories) {
+         my $directories = File::Spec->catdir(@directories);
+         for my $format (qw< json yaml >) {
+            my $path = File::Spec->catpath($volume, $directories, "environment.$format");
+            return _slurp($path) if -e $path;
+         }
+         pop @directories;
+      }
+   }
+
    return;
 }
 
+sub _find_code_dir {
+   return $main_dotcloud_code_dir if -d $main_dotcloud_code_dir;
+
+   my $n = shift || 0;
+   require Cwd;
+   require File::Basename;
+   require File::Spec;
+   for my $path (Cwd::cwd(), File::Basename::dirname((caller($n))[1])) {
+      my $abspath = File::Spec->file_name_is_absolute($path) ? $path : File::Spec->rel2abs($path);
+      my ($volume, $directories) = File::Spec->splitpath($abspath, 'no-file');
+      my @directories = File::Spec->splitdir($directories);
+      while (@directories) {
+         my $directories = File::Spec->catdir(@directories);
+         my $filepath = File::Spec->catpath($volume, $directories, 'dotcloud.yml');
+         return File::Spec->catpath($volume, $directories, '') if -e $filepath;
+         pop @directories;
+      }
+   }
+}
+
+sub find_code_dir {
+   my %params = (@_ > 0 && ref $_[0]) ? %{ $_[0] } : @_;
+   my $dir = _find_code_dir($params{n});
+   if (defined($dir) && $params{unix}) {
+      require File::Spec;
+      my $reldir = File::Spec->abs2rel($dir);
+      my @dirs = File::Spec->splitdir($reldir);
+      $dir = join '/', @dirs;
+   }
+   return $dir;
+}
+
 sub _dclone {
    return dclone(ref $_[0] ? $_[0] : {@_});
 }
@@ -216,6 +276,19 @@ sub service {
    _dclone_return(@found_services);
 }
 
+sub service_vars {
+   my $self = shift;
+   my %params = (@_ > 0 && ref($_[0])) ? %{$_[0]} : @_;
+   my $service = $self->service(@_);
+   if (exists $params{list}) {
+      my @list = @{$params{list}};
+      my @values = @{$service->{vars}}{@list};
+      return @values if wantarray;
+      return \@values;
+   }
+   _dclone_return($service->{vars});
+}
+
 1;
 
 
@@ -227,7 +300,7 @@ DotCloud::Environment - easy handling of environment in dotcloud
 
 =head1 VERSION
 
-version 0.7.0_02
+version 0.7.0_03
 
 =head1 SYNOPSIS
 
@@ -236,6 +309,9 @@ version 0.7.0_02
    # get an object, fallback to $path if not in dotCloud deploy
    my $dcenv = DotCloud::Environment->new(fallback_file => $path);
 
+   # even more lazy, make it look for files in various directories
+   $dcenv = DotCloud::Environment->new(backtrack => 1);
+
    # you should now which services make part of your stack!
    my $nosqldb_conf = $dcenv->service('nosqldb');
    my $type = $nosqldb_conf->{type}; # e.g. mysql, redis, etc.
@@ -337,6 +413,11 @@ use the provided string if other methods fail;
 
 use the provided file if other methods fail.
 
+=item B<< backtrack >>
+
+if nothing works and no fallback is set, look for suitable files
+in filesystem.
+
 =back
 
 Unless C<no_load> is passed and set to true, the object creation also
@@ -350,8 +431,9 @@ Returns the new object or C<croak>s if errors occur.
    $dcenv->load({%params});
 
 loads the configuration for an application. The accepted parameters are
-C<environment_string>, C<environment_file>, C<fallback_string> and
-C<fallback_file> with the same meaning as in the constructor (see L</new>).
+C<environment_string>, C<environment_file>, C<fallback_string>,
+C<fallback_file> and C<backtrack> with the same meaning as in the
+constructor (see L</new>).
 
 The sequence to get the configuration string is the following:
 
@@ -400,6 +482,28 @@ from parameter set in the constructor
 
 =back
 
+If none of the above works there's still some hope in case there is
+option C<backtrack> (or it was specified to the constructor). In this
+case, either file is searched recursively starting from the
+following directories:
+
+=over
+
+=item *
+
+the one returned by L</find_code_dir> (but as if it were called by
+the caller of L</load>, i.e. with a value of C<n> equal to 1)
+
+=item *
+
+the current working directory
+
+=item *
+
+the directory of the file that called us.
+
+=back
+
 It is possible to load multiple configuration files from
 multiple applications.
 
@@ -594,6 +698,75 @@ applications.
 If exactly one service is found it is returned, otherwise this method
 C<croak>s.
 
+=head2 service_vars
+
+   my %vars   = $dcenv->service_vars(%params); # also \%params
+   my $vars   = $dcenv->service_vars(%params); # also \%params
+   my @values = $dcenv->service_vars(%params); # also \%params
+   my $values = $dcenv->service_vars(%params); # also \%params
+
+this method is a shorthand to get the configuration variables of a single
+service. Depending on the input, the return value might be structured like
+a hash or like an array:
+
+=over
+
+=item B<< service >>
+
+the name of the service, see L</service>
+
+=item B<< application >>
+
+the name of the application, see L</service>
+
+=item B<< list >>
+
+(B<Optional>) if a list is provided, then the values corresponding to each
+item in order is returned. This allows writing things like this:
+
+   my ($host, $port, $password) = $dcenv->service_list(
+      service => 'nosqldb',
+      list => [ qw< host port password > ],
+   );
+
+and get directly the values to put into variables. In this case, the return
+value can be a list of values or an anonymous array with the values.
+
+If this parameter is not present, the whole name/value hash is returned, either
+as a list or as an anonymous hash depending on the context.
+
+=back
+
+=head2 find_code_dir
+
+   my $code_directory = $dcenv->find_code_dir(%params);
+   my $code_directory = DotCloud::Environment->find_code_dir(%params);
+   my $code_directory = DotCloud::Environment::find_code_dir(%params);
+
+not really a method, this function tries to find the file F<dotcloud.yml> that
+describe the application backtracking from the current working directory and
+from the directory containing the file that called us (i.e. what happens to
+be C<(caller($n))[1]>).
+
+Parameters:
+
+=over
+
+=item B<< n >>
+
+an integer, defaulting to 0, that tells how to call
+C<caller()>. You shouldn't need to set it, anyway.
+
+=item B<< unix >>
+
+when set, the name of the directory will be returned in Unix format, so that
+you can use it with C<use lib>.
+
+=back
+
+This should be useful if you want to put a default configuration file there or
+if you want to set up a shared library directory.
+
 =head1 AUTHOR
 
 Flavio Poletti <polettix@cpan.org>
